<?php

declare(strict_types=1);

namespace Drupal\role_revocation\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A form for role revocation settings.
 *
 * @package Drupal\role_revocation\Form
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'role_revocation_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function getEditableConfigNames() {
    return [
      'role_revocation.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('role_revocation.settings');

    $roles = $this->entityTypeManager->getStorage('user_role')->loadMultiple();
    unset($roles['anonymous'], $roles['authenticated']);

    foreach ($roles as $role) {
      $roleOptions[$role->id()] = $role->label();
    }

    $form['enable_cron'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable role revocation cron job'),
      '#default_value' => $config->get('enable_cron'),
    ];

    $form['roles'] = [
      '#type' => 'checkboxes',
      '#options' => $roleOptions,
      '#title' => $this->t('Roles to revoke'),
      '#default_value' => $config->get('roles') ?? [],
    ];

    $form['privileged_roles'] = [
      '#type' => 'checkboxes',
      '#options' => $roleOptions,
      '#title' => $this->t('Privileged roles'),
      '#description' => $this->t('Role revocation will not apply for any user with privileged roles. Users with privileged role will keep all their roles.'),
      '#default_value' => $config->get('privileged_roles') ?? [],
    ];

    $form['threshold'] = [
      '#type' => 'number',
      '#title' => $this->t('Role revocation threshold (days)'),
      '#default_value' => $config->get('threshold'),
      '#description' => $this->t('Enter the number of days after which roles should be revoked.'),
      '#min' => 1,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('role_revocation.settings');

    $config->set('roles', array_values(array_filter($form_state->getValue('roles') ?? [])))
      ->set('privileged_roles', array_values(array_filter($form_state->getValue('privileged_roles') ?? [])))
      ->set('enable_cron', (bool) $form_state->getValue('enable_cron'))
      ->set('threshold', (int) $form_state->getValue('threshold'));
    $config->save();

    parent::submitForm($form, $form_state);
  }

}
